#ifndef MSP_DATAFILE_BINFLOAT_H_
#define MSP_DATAFILE_BINFLOAT_H_

#include <climits>
#include <cstdint>
#include <cstring>

namespace Msp {
namespace DataFile {

/**
Facilitates splitting floating-point numbers into parts and putting them back
together.  Supports arbitary sizes up to 64 bits.  The 16, 32 and 64 bit
formats exactly match those defined by ISO/IEC 60559:2011.

The exponent is stored in an unbiased form.  The mantissa is stored with the
integer part included, aligned to the high bits of a 64-bit integer.
*/
struct BinFloat
{
	struct Bits
	{
		unsigned exponent;
		unsigned mantissa;

		Bits(unsigned);
	};

	template<typename T>
	struct MatchingInt;

	bool sign;
	bool infinity;
	int exponent;
	std::uint64_t mantissa;

	static BinFloat explode(std::uint64_t, const Bits &);

	template<typename T>
	static BinFloat explode_iec559(T v);

	std::uint64_t compose(const Bits &);

	template<typename T>
	T compose_iec559();
};

template<>
struct BinFloat::MatchingInt<float> { typedef std::uint32_t Type; };

template<>
struct BinFloat::MatchingInt<double> { typedef std::uint64_t Type; };

template<typename T>
inline BinFloat BinFloat::explode_iec559(T v)
{
	typename MatchingInt<T>::Type i;
	memcpy(&v, &i, sizeof(T));
	return explode(i, sizeof(T)*CHAR_BIT);
}

template<typename T>
inline T BinFloat::compose_iec559()
{
	typename MatchingInt<T>::Type i = compose(sizeof(T)*CHAR_BIT);
	T v;
	memcpy(&v, &i, sizeof(T));
	return v;
}

} // namespace DataFile
} // namespace Msp

#endif
