#ifndef MSP_DATAFILE_VALUE_H_
#define MSP_DATAFILE_VALUE_H_

#include <vector>
#include <msp/core/attributes.h>
#include <msp/core/meta.h>
#include <msp/core/variant.h>
#include "type.h"

namespace Msp {
namespace DataFile {

class Value
{
private:
	char sig;
	Variant data;

public:
	template<typename T>
	Value(T d):
		sig(TypeInfo<T>::signature),
		data(static_cast<typename TypeInfo<T>::Store>(d))
	{ }

	Value(Symbol d): sig(TypeInfo<Symbol>::signature), data(d) { }

	template<typename T>
	typename TypeInfo<T>::Load get() const
	{ return get_<typename TypeInfo<T>::Store>(); }

	char get_signature() const { return sig; }
private:
	template<typename T>
	T get_() const;
};

template<typename T>
inline T Value::get_() const
{
	return data.value<typename TypeInfo<T>::Store>();
}

template<>
inline FloatType::Store Value::get_<FloatType::Store>() const
{
	if(sig==IntType::signature)
		return data.value<IntType::Store>();
	else
		return data.value<FloatType::Store>();
}

} // namespace DataFile
} // namespace Msp

#endif
